@@ -31,6 +31,7 @@
  * Import pkg_summary to SQLite database
  */
 
+#include <sqlite3.h>
 #include "pkgin.h"
 
 static const struct Summary {
@@ -211,27 +212,30 @@ prepare_insert(int pkgid, struct Summary sum)
 	char		querybuf[4096];
 	char		tmpbuf[1024];
 
-	snprintf(querybuf, sizeof(querybuf), "INSERT INTO %s (PKG_ID", sum.tbl_name);
+	sqlite3_snprintf(sizeof(querybuf), querybuf, "INSERT INTO %w (PKG_ID",
+	    sum.tbl_name);
 
 	/* insert fields */
 	SLIST_FOREACH(pi, &inserthead, next) {
-		snprintf(tmpbuf, sizeof(tmpbuf), ",%s", pi->field);
-		if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >= sizeof(querybuf))
+		sqlite3_snprintf(sizeof(tmpbuf), tmpbuf, ",%w", pi->field);
+		if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >=
+		    sizeof(querybuf))
 			goto err;
 	}
 
-	snprintf(tmpbuf, sizeof(tmpbuf), ") VALUES (%d", pkgid);
+	sqlite3_snprintf(sizeof(tmpbuf), tmpbuf, ") VALUES (%d", pkgid);
 	if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >= sizeof(querybuf))
 		goto err;
 
 	/* insert values */
 	SLIST_FOREACH(pi, &inserthead, next) {
-		snprintf(tmpbuf, sizeof(tmpbuf), ",'%s'", pi->value);
-		if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >= sizeof(querybuf))
+		sqlite3_snprintf(sizeof(tmpbuf), tmpbuf, ",%Q", pi->value);
+		if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >=
+		    sizeof(querybuf))
 			goto err;
 	}
 
-	snprintf(tmpbuf, sizeof(tmpbuf), ");");
+	sqlite3_snprintf(sizeof(tmpbuf), tmpbuf, ");");
 	if (strlcat(querybuf, tmpbuf, sizeof(querybuf)) >= sizeof(querybuf))
 		goto err;
 
@@ -266,7 +270,7 @@ parse_entry(struct Summary sum, int pkgid, char *line)
 {
 	static uint8_t	check_machine_arch = 1;
 	int		i;
-	char		*val, *v, *p, *pkg, buf[BUFSIZ], *tmp;
+	char		*val, *v, *pkg, buf[BUFSIZ];
 
 	if ((val = strchr(line, '=')) == NULL)
 		errx(EXIT_FAILURE, "Invalid pkg_info entry: %s", line);
@@ -340,26 +344,6 @@ parse_entry(struct Summary sum, int pkgid, char *line)
 		snprintf(buf, BUFSIZ, "%s=", cols.name[i]);
 
 		if (strncmp(buf, line, strlen(buf)) == 0) {
-			tmp = NULL;
-			/*
-			 * Convert single quotes to SQLite compatible ''.
-			 */
-			if (strchr(val, '\'') != NULL) {
-				for (v = p = val; *v != '\0'; v++) {
-					if (*v != '\'')
-						continue;
-
-					*v = '\0';
-					if (tmp)
-						tmp = xasprintf("%s%s''", tmp, p);
-					else
-						tmp = xasprintf("%s''", p);
-					p = v + 1;
-				}
-				tmp = xasprintf("%s%s", tmp, p);
-				val = tmp;
-			}
-
 			/* Split PKGNAME into parts */
 			if (strncmp(cols.name[i], "PKGNAME", 7) == 0) {
 				/* some rare packages have no version */
@@ -378,8 +362,6 @@ parse_entry(struct Summary sum, int pkgid, char *line)
 				add_to_slist("PKGVERS", v);
 			} else
 				add_to_slist(cols.name[i], val);
-
-			free(tmp);
 			break;
 		}
 	}
@@ -393,6 +375,7 @@ insert_local_summary(FILE *fp)
 {
 	static int	pkgid = 1;
 	char		buf[BUFSIZ];
+	uint64_t	savepoint;
 
 	if (fp == NULL) {
 		pkgindb_close();
@@ -400,13 +383,13 @@ insert_local_summary(FILE *fp)
 	}
 
 	/* record columns names to cols */
-	snprintf(buf, BUFSIZ, "PRAGMA table_info(%s);",
-			sumsw[LOCAL_SUMMARY].tbl_name);
+	sqlite3_snprintf(BUFSIZ, buf, "PRAGMA table_info(%w);",
+	    sumsw[LOCAL_SUMMARY].tbl_name);
 	pkgindb_doquery(buf, colnames, NULL);
 
 	SLIST_INIT(&inserthead);
 
-        pkgindb_doquery("BEGIN;", NULL, NULL);
+	savepoint = pkgindb_savepoint();
 
 	while (fgets(buf, BUFSIZ, fp) != NULL) {
 		/*
@@ -421,7 +404,7 @@ insert_local_summary(FILE *fp)
 		parse_entry(sumsw[LOCAL_SUMMARY], pkgid, buf);
 	}
 
-        pkgindb_doquery("COMMIT;", NULL, NULL);
+	pkgindb_savepoint_release(savepoint);
 }
 
 /*
@@ -434,6 +417,7 @@ insert_remote_summary(struct archive *a, char *cur_repo)
 	size_t		buflen, offset;
 	ssize_t		r;
 	char		*buf, *pe, *pi, *npi;
+	uint64_t	savepoint;
 
 	if (a == NULL) {
 		pkgindb_close();
@@ -449,13 +433,13 @@ insert_remote_summary(struct archive *a, char *cur_repo)
 	buf = xmalloc(buflen + 1);
 
 	/* record columns names to cols */
-	snprintf(buf, buflen, "PRAGMA table_info(%s);",
+	sqlite3_snprintf(buflen, buf, "PRAGMA table_info(%w);",
 	    sumsw[REMOTE_SUMMARY].tbl_name);
 	pkgindb_doquery(buf, colnames, NULL);
 
 	SLIST_INIT(&inserthead);
 
-	pkgindb_doquery("BEGIN;", NULL, NULL);
+	savepoint = pkgindb_savepoint();
 
 	/*
 	 * Main loop.  Read in archive, split into package records and parse
@@ -526,14 +510,14 @@ insert_remote_summary(struct archive *a, char *cur_repo)
 
 	XFREE(buf);
 
-	pkgindb_doquery("COMMIT;", NULL, NULL);
-
 	if (r != ARCHIVE_OK) {
-		delete_remote_tbl(sumsw[REMOTE_SUMMARY], cur_repo);
+		pkgindb_savepoint_rollback(savepoint);
 		errx(EXIT_FAILURE, "Short read of pkg_summary: %s",
 		    archive_error_string(a));
 	}
 
+	pkgindb_savepoint_release(savepoint);
+
 #if ARCHIVE_VERSION_NUMBER < 3000000
 	archive_read_finish(a);
 #else
@@ -579,12 +563,31 @@ update_localdb(char **pkgkeep)
 	Plistnumbered	*keeplisthead, *nokeeplisthead;
 	Pkglist		*pkglist;
 
+	/*
+	 * Start a write transaction, excluding other writers until
+	 * committed.  This serves two purposes:
+	 *
+	 * - If interrupted before we've recreated the local summary
+	 *   and reinitialized the PKG_KEEP state, this ensures we
+	 *   don't lose anything.
+	 *
+	 * - If a concurrent pkgin process simultaneously tries to do
+	 *   update_localdb, it can't invalidate the keep list we read
+	 *   with KEEP_LOCAL_PKGS.
+	 *
+	 * Note that this does not nest inside transactions or
+	 * savepoints.
+	 */
+	if (pkgindb_doquery("BEGIN IMMEDIATE;", NULL, NULL))
+		errx(EXIT_FAILURE, "failed to begin immediate transaction");
+
 	/* has the pkgdb (pkgsrc) changed ? if not, continue */
 	if (!pkg_db_mtime(&st) && !force_fetch)
-		return;
+		goto out;
 
 	/* record the keep list */
 	keeplisthead = rec_pkglist(KEEP_LOCAL_PKGS);
+
 	/* delete local pkg table (faster than updating) */
 	pkgindb_doquery(DELETE_LOCAL, NULL, NULL);
 
@@ -644,6 +647,9 @@ update_localdb(char **pkgkeep)
 	if (pkgkeep != NULL)
 		/* installation: mark the packages as "keep" */
 		pkg_keep(KEEP, pkgkeep);
+
+out:	if (pkgindb_doquery("COMMIT;", NULL, NULL))
+		errx(EXIT_FAILURE, "failed to commit transaction");
 }
 
 static int
